/*  Copyright 2012 Bronwyn Woods
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package com.dancingwoods.choreo;

import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

/**
 * 
 * Class to specify the SQLite data structure
 * 
 * @author Bronwyn Woods
 *
 */
public class DanceDb extends SQLiteOpenHelper{
    private static final String TAG = "DanceDb";
    private static final int DATABASE_VERSION = 12;

	
	/**
	 * Database fields
	 */
    public static final String AUTH_FNAME = "fname";
    public static final String AUTH_LNAME = "lname";
    public static final String AUTH_WEB = "website";
    public static final String AUTH_EMAIL = "email";
    public static final String AUTH_ROWID = "_id";
    
    public static final String MOVE_DESC = "description";
    public static final String MOVE_COUNT = "count";
    public static final String MOVE_SHORT1 = "short1";
    public static final String MOVE_SHORT2 = "short2";
    public static final String MOVE_ROWID = "_id";
    
    public static final String TAG_DESC = "description";
    public static final String TAG_CAT = "category";
    public static final String TAG_ROWID = "_id";
    
    public static final String PROGRAM_DATE = "date";
    public static final String PROGRAM_NOTES = "notes";
    public static final String PROGRAM_ROWID = "_id";
    
    public static final String DANCE_NAME = "name";
    public static final String DANCE_DIFF = "difficulty";
    public static final String DANCE_RATING = "rating";
    public static final String DANCE_FORMATION = "formation";
    public static final String DANCE_PROGRESSION = "progression";
    public static final String DANCE_NOTES = "notes";
    public static final String DANCE_NOTESCALL = "notescall";
    public static final String DANCE_NOTESMUSIC = "notesmusic";
    public static final String DANCE_GENRE = "genre";
    public static final String DANCE_ROWID = "_id";
    
    public static final String SERIES_NAME = "name";
    public static final String SERIES_LOCATION = "location";
    public static final String SERIES_NOTES = "notes";
    public static final String SERIES_ROWID = "_id";
    
    public static final String BAND_NAME = "name";
    public static final String BAND_NOTES = "notes";
    public static final String BAND_ROWID = "_id";
    
    public static final String DANCEAUTHOR_DANCEID = "danceid";
    public static final String DANCEAUTHOR_AUTHORID = "authorid";
    public static final String DANCEAUTHOR_ROWID = "_id";
    
    public static final String MOVETAG_MOVEID = "moveid";
    public static final String MOVETAG_TAGID = "tagid";
    public static final String MOVETAG_ROWID = "_id";

    public static final String DANCEPROGRAM_DANCEID = "danceid";
    public static final String DANCEPROGRAM_PROGRAMID = "programid";
    public static final String DANCEPROGRAM_ORDER = "danceorder";
    public static final String DANCEPROGRAM_ALT = "alt";
    public static final String DANCEPROGRAM_BREAK = "break";
    public static final String DANCEPROGRAM_ROWID = "_id";
    
    public static final String DANCETAG_DANCEID = "danceid";
    public static final String DANCETAG_TAGID = "tagid";
    public static final String DANCETAG_ROWID = "_id";
    
    public static final String DANCEMOVE_DANCEID = "danceid";
    public static final String DANCEMOVE_MOVEID = "moveid";
    public static final String DANCEMOVE_ORDER = "moveorder";
    public static final String DANCEMOVE_SECTION = "section";
    public static final String DANCEMOVE_NOTE = "note";
    public static final String DANCEMOVE_ROWID = "_id";
    
    public static final String PROGRAMSERIES_PROGRAMID = "programid";
    public static final String PROGRAMSERIES_SERIESID = "seriesid";
    public static final String PROGRAMSERIES_ROWID = "_id";
    
    public static final String PROGRAMBAND_PROGRAMID = "programid";
    public static final String PROGRAMBAND_BANDID = "bandid";
    public static final String PROGRAMBAND_ROWID = "_id";

    /**
     * Database tables
     */
    public static final String DATABASE_NAME = "dancedata";

    public static final String TABLE_AUTHORS = "authors";
    public static final String TABLE_MOVES = "moves";
    public static final String TABLE_TAGS = "tags";
    public static final String TABLE_DANCES = "dances";
    public static final String TABLE_PROGRAMS = "programs";
    public static final String TABLE_SERIES = "series";
    public static final String TABLE_BANDS = "bands";

    public static final String TABLE_DANCEAUTHOR = "danceauthor";
    public static final String TABLE_DANCEPROGRAM = "danceprogram";
    public static final String TABLE_DANCETAG = "dancetag";
    public static final String TABLE_DANCEMOVE = "dancemove";
    public static final String TABLE_MOVETAG = "movetag";
    public static final String TABLE_PROGRAMSERIES = "programseries";
    public static final String TABLE_PROGRAMBAND = "programband";
    
    /**
     * Database creation sql statements
     */
    private static final String AUTHORS_CREATE =
	        "create table authors (_id integer primary key autoincrement, " +
	        "fname text not null," +
	        "lname text not null," +
	        "website text,"+
	        "email text"+
	        ");";  
    private static final String MOVES_CREATE =
            "create table moves (_id integer primary key autoincrement, " +
            "description text not null," +
            "count integer not null," +
            "short1 text,"+
            "short2 text"+
            ");";    
    private static final String TAGS_CREATE =
            "create table tags (_id integer primary key autoincrement, " +
            "description text not null," +
            "category text not null" +		
            ");";   
    private static final String DANCES_CREATE =
            "create table dances (_id integer primary key autoincrement, " +
            "name text not null," +
            "formation text,"+
            "difficulty integer," +
            "rating integer,"+
            "progression text,"+
            "notes text,"+
            "notescall text,"+
            "notesmusic text,"+
            "genre text"+
            ");";
    private static final String PROGRAMS_CREATE =
            "create table programs (_id integer primary key autoincrement, " +
            "name text," +
            "date text," +
            "notes text"+
            ");";
    private static final String SERIES_CREATE =
            "create table series (_id integer primary key autoincrement, " +
            "name text," +
            "location text," +
            "notes text"+
            ");";
    private static final String BANDS_CREATE =
            "create table bands (_id integer primary key autoincrement, " +
            "name text," +
            "notes text"+
            ");";
    private static final String DANCEAUTHOR_CREATE =
            "create table danceauthor (_id integer primary key autoincrement, " +
            "danceid integer not null," +
            "authorid integer not null" +
            ");";   
    private static final String DANCETAG_CREATE =
            "create table dancetag (_id integer primary key autoincrement, " +
            "danceid integer not null," +
            "tagid integer not null" +
            ");";
    private static final String MOVETAG_CREATE =
            "create table movetag (_id integer primary key autoincrement, " +
            "moveid integer not null," +
            "tagid integer not null" +
            ");"; 
    private static final String PROGRAMSERIES_CREATE =
            "create table programseries (_id integer primary key autoincrement, " +
            "programid integer not null," +
            "seriesid integer not null" +
            ");";   
    private static final String PROGRAMBAND_CREATE =
            "create table programband (_id integer primary key autoincrement, " +
            "programid integer not null," +
            "bandid integer not null" +
            ");";
    private static final String DANCEMOVE_CREATE =
            "create table dancemove (_id integer primary key autoincrement, " +
            "danceid integer not null," +
            "moveid integer not null," +
            "moveorder integer not null," +
            "section text" +            
            ");";   
    private static final String DANCEPROGRAM_CREATE =
            "create table danceprogram (_id integer primary key autoincrement, " +
            "danceid integer not null," +
            "programid integer not null," +
            "danceorder integer not null," +
            "alt integer," +
            "break integer"+
            ");";



    DanceDb(Context context) {
        super(context, DATABASE_NAME, null, DATABASE_VERSION);
    }

    @Override
    public void onCreate(SQLiteDatabase db) {
        db.execSQL(AUTHORS_CREATE);
        db.execSQL(DANCES_CREATE);
        db.execSQL(MOVES_CREATE);
        db.execSQL(TAGS_CREATE);
        db.execSQL(PROGRAMS_CREATE);
        db.execSQL(BANDS_CREATE);
        db.execSQL(SERIES_CREATE);
        db.execSQL(DANCEMOVE_CREATE);
        db.execSQL(DANCEPROGRAM_CREATE);
        db.execSQL(DANCETAG_CREATE);
        db.execSQL(DANCEAUTHOR_CREATE);
        db.execSQL(MOVETAG_CREATE);
        db.execSQL(PROGRAMSERIES_CREATE);
        db.execSQL(PROGRAMBAND_CREATE);
    }

    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
        Log.w(TAG, "Upgrading database from version " + oldVersion + " to "
                + newVersion + ", which will destroy all old data");
        db.execSQL("DROP TABLE IF EXISTS authors");
        db.execSQL("DROP TABLE IF EXISTS dances");
        db.execSQL("DROP TABLE IF EXISTS moves");
        db.execSQL("DROP TABLE IF EXISTS tags");
        db.execSQL("DROP TABLE IF EXISTS programs");
        db.execSQL("DROP TABLE IF EXISTS series");
        db.execSQL("DROP TABLE IF EXISTS bands");
        db.execSQL("DROP TABLE IF EXISTS movetag");
        db.execSQL("DROP TABLE IF EXISTS danceauthor");
        db.execSQL("DROP TABLE IF EXISTS dancemove");
        db.execSQL("DROP TABLE IF EXISTS dancetag");
        db.execSQL("DROP TABLE IF EXISTS danceprogram");
        db.execSQL("DROP TABLE IF EXISTS programseries");
        db.execSQL("DROP TABLE IF EXISTS programband");
        onCreate(db);
    }
}