/*  Copyright 2012 Bronwyn Woods
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package com.dancingwoods.choreo;

import java.util.ArrayList;

import android.app.ListFragment;
import android.app.LoaderManager;
import android.content.ContentValues;
import android.content.CursorLoader;
import android.content.Loader;
import android.database.Cursor;
import android.os.Bundle;
import android.util.Log;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.AutoCompleteTextView;
import android.widget.Spinner;

/**
 * Fragment to handle entering basic information about a
 * dance (title, author(s), formation, rating, ...)
 * 
 * @author Bronwyn Woods
 *
 */
public class FragDanceEditInfo extends ListFragment implements LoaderManager.LoaderCallbacks<Cursor>{
		
	//Local handles on the data entry fields
	//private EditText mTitle;
	private AutoCompleteTextView mForm;
	private AutoCompleteTextView mProg;
	private AutoCompleteTextView mAuthEnter;
	private Spinner mDifficulty;
	//private RatingBar mRating;
	
    private static final int DELETE_ID = Menu.FIRST+1;
	
	//String to hold currently selected authors for list view
    private ArrayList<String> mAuthListItems=new ArrayList<String>();
    private ArrayAdapter<String> mAuthAdapter;

    //Adaptor for difficulty spinner
    private ArrayAdapter<CharSequence> mDiffAdapter;
   
    //Author autocomplete adapter
    private ArrayAdapter<String> mAuthAdapt;
    private ArrayList<String> mAuthOptions=new ArrayList<String>();

    //Formation autocomplete adapter
    private ArrayAdapter<String> mFormAdapt;
    private ArrayList<String> mFormOptions=new ArrayList<String>();
    
    //Progression autocomplete adapter
    private ArrayAdapter<String> mProgAdapt;
    private ArrayList<String> mProgOptions=new ArrayList<String>();
    
    //private DanceInfoManipListener mInfoListener;

    // Container Activity must implement this interface
    public interface DanceInfoManipListener {
        public void onDanceInfoUpdate(Long mRowID, ContentValues values);
    }
    

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
    	return inflater.inflate(R.layout.frag_dance_edit_info, container, false);        
    }
  
    @Override
    public void onActivityCreated(Bundle savedInstanceState){
    	super.onActivityCreated(savedInstanceState);
    	
        getLoaderManager().initLoader(0, null, this);
        getLoaderManager().initLoader(1, null, this);
        getLoaderManager().initLoader(2, null, this);
    	        
        // Set the adaptor for the difficulty spinner, set local variable for difficulty
        mDifficulty = (Spinner) getView().findViewById(R.id.dance_edit_diff);
        mDiffAdapter = ArrayAdapter.createFromResource(
                getActivity(), R.array.diff_array, android.R.layout.simple_spinner_item);
        mDiffAdapter.setDropDownViewResource(R.layout.row_dropdown);
        mDifficulty.setAdapter(mDiffAdapter);
        
    	// Set the author list to use to the adaptor with the list items given
        mAuthAdapter=new ArrayAdapter<String>(getActivity(), android.R.layout.simple_list_item_1, mAuthListItems);
        setListAdapter(mAuthAdapter);

        // Author autocomplete fields
        mAuthEnter = (AutoCompleteTextView) getView().findViewById(R.id.dance_add_auth);
        mAuthAdapt = new ArrayAdapter<String>(getActivity(),
                R.layout.row_dropdown, mAuthOptions);
        mAuthEnter.setAdapter(mAuthAdapt);
        //mAuthEnter.setOnEditorActionListener(this);
 
        // Progression autocomplete fields
        mProg = (AutoCompleteTextView) getView().findViewById(R.id.dance_progression_edit);
        mProgAdapt = new ArrayAdapter<String>(getActivity(),
                R.layout.row_dropdown, mProgOptions);
        mProg.setAdapter(mProgAdapt);
        //mProg.setOnEditorActionListener(this);

        // Formation autocomplete
        mForm = (AutoCompleteTextView) getView().findViewById(R.id.dance_formation_edit);
        mFormAdapt = new ArrayAdapter<String>(getActivity(),
                R.layout.row_dropdown, mFormOptions);
        mForm.setAdapter(mFormAdapt);
        //mForm.setOnEditorActionListener(this);
    
    }
    
    /**
     * Context menu appears when a list item is clicked and held
     */
    @Override
	public void onCreateContextMenu(ContextMenu menu, View v,
			ContextMenuInfo menuInfo) {
	    super.onCreateContextMenu(menu, v, menuInfo);
	    menu.add(0, DELETE_ID, 0, "Remove Author");
	}

    
    /**
     * Handle context menu selection.
     * Call the appropriate callback method in the container activity
     */
    @Override
	public boolean onContextItemSelected(MenuItem item) {
        switch(item.getItemId()) {
        case DELETE_ID:
        	mAuthListItems.remove(item);
        	mAuthAdapter.notifyDataSetChanged();
        	break;
        }
        return super.onContextItemSelected(item);
	}
    

    /**
     * Adds an author's name to the list of authors displayed in this fragment
     * @param s the author's name
     */
	public void addAuthor(String s) {
		mAuthListItems.add(s);
		mAuthAdapter.notifyDataSetChanged();
    }
	
//	@Override
//	public boolean onEditorAction(TextView view, int actionID, KeyEvent kevent) {
//		// TODO Auto-generated method stub
//		return false;
//	}

	
    /** 
     * Management of data loader.  Specifies fields to be retrieved from the database.
     */
    @Override
    public Loader<Cursor> onCreateLoader(int id, Bundle args) {
    	CursorLoader cursorLoader=null;
    	if(id==0){
	    	String[] projection = {DanceDb.AUTH_ROWID,DanceDb.AUTH_FNAME, DanceDb.AUTH_LNAME};
	        cursorLoader = new CursorLoader(getActivity(),
	                DanceProvider.AUTHORS_URI, projection, null, null, DanceDb.AUTH_LNAME);
    	}else if(id==1){
    		String[] projection = {DanceDb.DANCE_FORMATION};
    		cursorLoader = new CursorLoader(getActivity(), DanceProvider.DANCES_UNIQUE_URI, 
    				projection, null, null, DanceDb.DANCE_FORMATION);
    	}else if(id==2){
    		String[] projection = {DanceDb.DANCE_PROGRESSION};
    		cursorLoader = new CursorLoader(getActivity(), DanceProvider.DANCES_UNIQUE_URI, 
    				projection, null, null, DanceDb.DANCE_PROGRESSION);
    	}
        return cursorLoader;
    }
    
    /**
     * Management of data loader.  Set fragment fields to the database values.
     */
    @Override
    public void onLoadFinished(Loader<Cursor> loader, Cursor cursor) {
    	Integer id = loader.getId();
    	Log.d("DanceEditFragLoaderID", String.valueOf(id));
    	if(id==0){
	    	for(cursor.moveToFirst(); !cursor.isAfterLast(); cursor.moveToNext()) {
	    	    mAuthOptions.add(cursor.getString(
	                    cursor.getColumnIndexOrThrow(DanceDb.AUTH_FNAME)) + " " + cursor.getString(
	                            cursor.getColumnIndexOrThrow(DanceDb.AUTH_LNAME)));
	    	}
	    	mAuthAdapt.notifyDataSetChanged();
    	}else if(id==1){
	    	for(cursor.moveToFirst(); !cursor.isAfterLast(); cursor.moveToNext()) {
	    	    mFormOptions.add(cursor.getString(cursor.getColumnIndexOrThrow(DanceDb.DANCE_FORMATION)));
	    	}
	    	mFormAdapt.notifyDataSetChanged();
    	}else if(id==2){
	    	for(cursor.moveToFirst(); !cursor.isAfterLast(); cursor.moveToNext()) {
	    	    mProgOptions.add(cursor.getString(cursor.getColumnIndexOrThrow(DanceDb.DANCE_PROGRESSION)));
	    	}
	    	mProgAdapt.notifyDataSetChanged();
    	}
    }
 
    @Override
    public void onLoaderReset(Loader<Cursor> loader) {
        //mAuthAdapt.swapCursor(null);
    }

    
}


