/*  Copyright 2012 Bronwyn Woods
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package com.dancingwoods.choreo;

import android.app.Activity;
import android.app.ListFragment;
import android.app.LoaderManager;
import android.content.CursorLoader;
import android.content.Loader;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ListView;
import android.widget.SimpleCursorAdapter;
import android.widget.TextView;

/**
 * Fragment to hold fields for entering information about a move.
 * 
 * @author Bronwyn Woods
 *
 */
public class FragMoveView extends ListFragment implements LoaderManager.LoaderCallbacks<Cursor>{

    private MoveManipListener mMoveListener;

    private TextView mDescText;
    private TextView mSh1Text;
    private TextView mSh2Text;
    private TextView mTagsText;
    private Long mRowId;
    private SimpleCursorAdapter mDanceAdapt;
    private ListView mDanceList;
    
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setHasOptionsMenu(true);
	}

   /**
    * Interface that the container activity must implement.
    * onMoveUpdate handles saving an author after editing.
    */
    public interface MoveManipListener {
        public void onMoveEdit(Uri moveUri);
    }

    @Override
    public void onAttach(Activity activity) {
        super.onAttach(activity);
        try {
            mMoveListener = (MoveManipListener) activity;
        } catch (ClassCastException e) {
            throw new ClassCastException(activity.toString() + " must implement MoveManipListener method onMoveEdit");
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        return inflater.inflate(R.layout.frag_move_view, container, false);
    }
    
 
    @Override    
    public void onActivityCreated(Bundle savedInstanceState){
    	super.onActivityCreated(savedInstanceState);
    	
        mDescText = (TextView) getView().findViewById(R.id.move_view_desc);
        mSh1Text = (TextView) getView().findViewById(R.id.move_view_sh1);
        mSh2Text = (TextView) getView().findViewById(R.id.move_view_sh2);   
        mTagsText = (TextView) getView().findViewById(R.id.move_view_tags);   
        mDanceList = (ListView) getListView();
        
        // Create an array to specify the fields we want to display in the list
        String[] from = new String[]{DanceDb.DANCE_NAME};

        // and an array of the fields we want to bind those fields to (in this case just text1)
        int[] to = new int[]{R.id.dance_list_title};

        // Now create a simple cursor adapter and set it to display
        mDanceAdapt = 
            new SimpleCursorAdapter(getActivity(), R.layout.row_dance_list, null, from, to, 0);
        setListAdapter(mDanceAdapt);     
     }
    
	@Override
	public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
	    inflater.inflate(R.menu.move_view_menu, menu);
	}

	
    @Override
    public boolean onOptionsItemSelected(MenuItem item){
        switch(item.getItemId()) {
        case R.id.move_view_edit:
        	mMoveListener.onMoveEdit(Uri.withAppendedPath(DanceProvider.MOVES_URI, String.valueOf(mRowId)));
            return true;
        }
        return true;
    }

    
    public void displayMove(long rowid){
    	/*
    	 * Init the loader if there isn't already one assigned.
    	 * If there is already a loader, restart it to show the selected move
    	 */
        if(mRowId != null){
        	mRowId = rowid;
        	getLoaderManager().restartLoader(0, null, this);
        	getLoaderManager().restartLoader(1, null, this);
        	getLoaderManager().restartLoader(2, null, this);
        }else{
           	mRowId = rowid;
            // Loader to get move data
            getLoaderManager().initLoader(0, null, this);
            // Loader to get dance list
            getLoaderManager().initLoader(1, null, this);
            // Loader to get tag list
            getLoaderManager().initLoader(2, null, this);
        }
    }
    
    
    /*
     * Methods for loader management 
     */
    @Override
    public Loader<Cursor> onCreateLoader(int id, Bundle args) {
    	CursorLoader cursorLoader=null;
    	if(id==0){
    		// Get move data
	        String[] projection = {DanceDb.MOVE_ROWID, DanceDb.MOVE_COUNT, DanceDb.MOVE_DESC, DanceDb.MOVE_SHORT1, DanceDb.MOVE_SHORT2};
	        cursorLoader = new CursorLoader(getActivity(),
	                Uri.withAppendedPath(DanceProvider.MOVES_URI, String.valueOf(mRowId)), projection, null, null, null);
    	}else if(id==1){
    		// Get list of dances
    	}else if(id==2){
    		// Get tags for this move
    		String[] projection = {DanceDb.MOVETAG_MOVEID, DanceDb.TABLE_TAGS+"."+DanceDb.TAG_DESC};
    		cursorLoader = new CursorLoader(getActivity(),
    				Uri.parse(DanceProvider.TAGS_URI+"/move/"+String.valueOf(mRowId)), projection, null, null, null);
    	}
        return cursorLoader;
    }
 
    @Override
    public void onLoadFinished(Loader<Cursor> loader, Cursor cursor) {
    	Integer id = loader.getId();
     	if(id==0){
     		// Data for move
	    	cursor.moveToFirst();
	    	mDescText.setText("("+cursor.getString(
	                cursor.getColumnIndexOrThrow(DanceDb.MOVE_COUNT))+
	                ") "+cursor.getString(
	                cursor.getColumnIndexOrThrow(DanceDb.MOVE_DESC)));
	    	mSh1Text.setText(cursor.getString(
	                cursor.getColumnIndexOrThrow(DanceDb.MOVE_SHORT1)));
	    	mSh2Text.setText(cursor.getString(
	                cursor.getColumnIndexOrThrow(DanceDb.MOVE_SHORT2)));
    	}else if(id==1){
    	}else if(id==2){
    		// tags for this move
    		String taglist = "";
    		cursor.moveToFirst();
    		while(!cursor.isAfterLast()){
    			taglist = taglist+"  "+cursor.getString(
    	                cursor.getColumnIndexOrThrow(DanceDb.TABLE_TAGS+"."+DanceDb.TAG_DESC));
    			cursor.moveToNext();
    		}
    		mTagsText.setText(taglist);
    	}
    }
 
    @Override
    public void onLoaderReset(Loader<Cursor> loader) {
    	Integer id = loader.getId();
    	if(id==0){
    		// Loader for move data
	        mDescText.setText("");
	        mSh1Text.setText("");
	        mSh2Text.setText("");
    	}else if(id==1){
    		// Loader for dance list
    	}else if(id==2){
    		// Loader for tag list
    	}
    }

    
        
}
