/*  Copyright 2012 Bronwyn Woods
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package com.dancingwoods.choreo;

import android.app.Activity;
import android.app.ListFragment;
import android.app.LoaderManager;
import android.content.CursorLoader;
import android.content.Loader;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView.AdapterContextMenuInfo;
import android.widget.ListView;
import android.widget.SimpleCursorAdapter;
import android.widget.TextView;

/**
 * Fragment to display list of tags currently in the database.
 * @author Bronwyn Woods
 *
 */
public class FragTagList extends ListFragment implements LoaderManager.LoaderCallbacks<Cursor>{

    private static final int EDIT_ID = Menu.FIRST+1;
    private static final int DELETE_ID = Menu.FIRST+2;
    
    private SimpleCursorAdapter mTagAdapt;
    private TagManipListener mTagListener;
    //private EditText mNewTagEdit;
    
    /**
     * Interface that the container activity must implement.
     * onTagSelected handles showing a tags dances and moves.
     * onTagAdded handles adding a tag.
     */
     public interface TagManipListener {
         public void onTagSelected(Uri tagUri);
         public void onTagEdit(Uri tagUri);
         public void onTagDelete(Uri tagUri);
         //public void onTagAdded(String tagtext, String tagcat);
         public void onTagNew();
     }

    @Override
    public void onAttach(Activity activity) {
        super.onAttach(activity);
        try {
            mTagListener = (TagManipListener) activity;
        } catch (ClassCastException e) {
            throw new ClassCastException(activity.toString() + " must implement TagManipListener");
        }
    }
 
    @Override    
    public void onActivityCreated(Bundle savedInstanceState){
    	super.onActivityCreated(savedInstanceState);
    	
        registerForContextMenu(getListView());
        
//        mNewTagEdit = (EditText) getView().findViewById(R.id.addtag);
//        mNewTagEdit.setOnKeyListener(new View.OnKeyListener() {
//            public boolean onKey(View v, int keyCode, KeyEvent event) {
//                if (event.getAction() == KeyEvent.ACTION_DOWN) {
//                    if (keyCode == KeyEvent.KEYCODE_ENTER) {
//                        mTagListener.onTagAdded(String.valueOf(mNewTagEdit.getText()));
//                        mNewTagEdit.setText("");
//                        return true;
//                    }
//                }
//                return false;
//            }
//        });
//       
        getLoaderManager().initLoader(0, null, this);
         
        // Create an array to specify the fields we want to display in the list
        String[] from = new String[]{DanceDb.TAG_DESC, DanceDb.TAG_CAT};

        // and an array of the fields we want to bind those fields to (in this case just text1)
        int[] to = new int[]{R.id.tagrow1, R.id.tagcat};

        // Now create a simple cursor adapter and set it to display
        mTagAdapt = 
            new SimpleCursorAdapter(getActivity(), R.layout.row_tag_list, null, from, to, 0);
        mTagAdapt.setViewBinder(new SimpleCursorAdapter.ViewBinder() {
        	  @Override
        	  public boolean setViewValue(View view, Cursor cursor, int columnIndex) {
        		  	int catind = cursor.getColumnIndexOrThrow(DanceDb.TAG_CAT);
					if(catind == columnIndex && view.getId()==R.id.tagcat){
	        		  	
						TextView tview = (TextView)view;
						tview.setText(cursor.getString(columnIndex));
	        		  	String thisCat = cursor.getString(columnIndex);
						String prevCat = null;
						
						// previous row, for comparison
						if (cursor.getPosition() > 0 && cursor.moveToPrevious()) {
						    prevCat = cursor.getString(columnIndex);
						    cursor.moveToNext();
						}
						
						// enable section heading if it's the first one, or 
						// different from the previous one
						if (prevCat == null || !prevCat.equals(thisCat)) {
						    tview.setVisibility(View.VISIBLE);
						} else {
						    tview.setVisibility(View.GONE);
						
						}
						
						return true;
					}
					return false;
        	  }
       	});
        setListAdapter(mTagAdapt);
    }
    
    @Override
    public void onResume(){
    	super.onResume();
    	getLoaderManager().restartLoader(0,null, this);
    }
    
    public void notifyDataChange(){
    	getLoaderManager().restartLoader(0,null, this);
    }
        
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.frag_tag_list, container, false);
    }
    

    @Override
	public void onCreateContextMenu(ContextMenu menu, View v,
			ContextMenuInfo menuInfo) {
	    super.onCreateContextMenu(menu, v, menuInfo);
	    menu.add(0, DELETE_ID, 0, "Delete Tag");
	    menu.add(0, EDIT_ID, 0, "Edit Tag");
	}
    
    @Override
	public boolean onContextItemSelected(MenuItem item) {
        AdapterContextMenuInfo info = (AdapterContextMenuInfo) item.getMenuInfo();
        switch(item.getItemId()) {
        case DELETE_ID:
        	mTagListener.onTagDelete(Uri.withAppendedPath(DanceProvider.TAGS_URI, String.valueOf(info.id)));
        	break;
        case EDIT_ID:
        	mTagListener.onTagEdit(Uri.withAppendedPath(DanceProvider.TAGS_URI, String.valueOf(info.id)));
        	break;
        }
        return super.onContextItemSelected(item);
	}
 
    
    @Override
	public void onListItemClick(ListView l, View v, int position, long id) {
        super.onListItemClick(l, v, position, id);
        mTagListener.onTagSelected(Uri.withAppendedPath(DanceProvider.TAGS_URI, String.valueOf(id)));
    }


    /** Methods for loader management*/
    @Override
    public Loader<Cursor> onCreateLoader(int id, Bundle args) {
        String[] projection = {DanceDb.TAG_ROWID, DanceDb.TAG_CAT, DanceDb.TAG_DESC};
        
        CursorLoader cursorLoader = new CursorLoader(getActivity(),
                DanceProvider.TAGS_URI, projection, null, null, DanceDb.TAG_CAT+", "+DanceDb.TAG_DESC);
        return cursorLoader;
    }
 
    @Override
    public void onLoadFinished(Loader<Cursor> loader, Cursor cursor) {
        mTagAdapt.swapCursor(cursor);
    }
 
    @Override
    public void onLoaderReset(Loader<Cursor> loader) {
        mTagAdapt.swapCursor(null);
    }
    
}
