/*  Copyright 2012 Bronwyn Woods
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package com.dancingwoods.choreo;

import android.app.ActionBar;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.FragmentManager;
import android.app.FragmentTransaction;
import android.content.ContentResolver;
import android.content.ContentValues;
import android.content.DialogInterface;
import android.content.Intent;
import android.database.sqlite.SQLiteConstraintException;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.widget.ArrayAdapter;

/**
 * Main activity for dealing with authors in the database.
 * 
 * @author Bronwyn Woods
 *
 */
public class TabAuthor extends Activity implements FragAuthorView.AuthorManipListener, 
	FragAuthorList.AuthorManipListener, FragAuthorEdit.AuthorManipListener, ActionBar.OnNavigationListener{
	
	private ContentResolver mCr;
	
	private Long mRowId;
    private FragAuthorEdit mEditFrag;
    private FragAuthorView mViewFrag;
    private FragAuthorList mListFrag;
    private FragmentManager mFragManage;
    private ArrayAdapter<CharSequence> mNavSpin;

	
	
	@Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        mCr = getContentResolver();
        setContentView(R.layout.tab_author);
        mFragManage = getFragmentManager();

        mListFrag = (FragAuthorList) mFragManage.findFragmentById(R.id.authorlistfrag);
        mEditFrag = null;
 
        // Set up the dropdown navigation menu and set the current selection to authors
        // Use the ActionBar navigation
        mNavSpin = ArrayAdapter.createFromResource(this, R.array.nav_array, 
        		android.R.layout.simple_spinner_item);
        mNavSpin.setDropDownViewResource(R.layout.dropdown_nav);
        ActionBar actionBar = getActionBar();
        actionBar.setNavigationMode(ActionBar.NAVIGATION_MODE_LIST);
        actionBar.setListNavigationCallbacks(mNavSpin, this);
        actionBar.setSelectedNavigationItem(2);
     }
	
	@Override
	public void onResume(){
		super.onResume();
        ActionBar actionBar = getActionBar();
        actionBar.setSelectedNavigationItem(2);
	}
	
	
	/**
	 * Options menu
	 * new - action bar
	 */
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
	    MenuInflater inflater = getMenuInflater();
	    inflater.inflate(R.menu.tab_author, menu);
	    return true;
	}
    
	/**
	 * Deal with selections from options menu
	 * new - call new activity to enter author
	 */
    @Override
    public boolean onOptionsItemSelected(MenuItem item){
    	//Log.d("OptionsMenu", String.valueOf(item));
        switch(item.getItemId()) {
        case R.id.author_new:
            onAuthorNew();
            return true;
//        case android.R.id.home:
//	    	// app icon in action bar clicked; bring 
// 	        Intent intent = new Intent(this, ChoreoActivity.class);
//	        intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
//	        startActivity(intent);
//	        return true;
        }	
        return false;
    }

    /**
     * Callback method to handle selection from author list
     */
	@Override
	public void onAuthorSelected(Uri authorUri) {
		if(mViewFrag!=null){
			FragmentTransaction fTran = mFragManage.beginTransaction();
			fTran.remove(mViewFrag);
			fTran.commit();
			mViewFrag=null;
		}
		if(mEditFrag!=null){
			FragmentTransaction fTran = mFragManage.beginTransaction();
			fTran.remove(mEditFrag);
			fTran.commit();
			mEditFrag=null;
		}

		FragmentTransaction fTran = mFragManage.beginTransaction();
		FragAuthorView viewfrag = new FragAuthorView();
    	mRowId = Long.parseLong(authorUri.getLastPathSegment());
    	mViewFrag = viewfrag;
    	fTran.add(R.id.tabauthorfragholder, viewfrag, "viewfrag");
		fTran.commit();
		mFragManage.executePendingTransactions();
    	mViewFrag.displayAuthor(mRowId);
	}

	/**
	 * Callback method to handle edit selected from context menu
	 */
	@Override
	public void onAuthorEdit(Uri authorUri) {
		if(mEditFrag!=null){
			FragmentTransaction fTran = mFragManage.beginTransaction();
			fTran.remove(mEditFrag);
			fTran.commit();
			mEditFrag=null;
		}
		if(mViewFrag!=null){
			FragmentTransaction fTran = mFragManage.beginTransaction();
			fTran.remove(mViewFrag);
			fTran.commit();
			mViewFrag=null;
		}

		FragmentTransaction fTran = mFragManage.beginTransaction();
		FragAuthorEdit editfrag = new FragAuthorEdit();
    	mRowId = Long.parseLong(authorUri.getLastPathSegment());
    	mEditFrag = editfrag;
    	fTran.add(R.id.tabauthorfragholder, editfrag, "editfrag");
		fTran.commit();
		mFragManage.executePendingTransactions();
    	mEditFrag.displayAuthor(mRowId);
 	}
	

	/**
	 * Callback method to handle delete from context menu
	 */
	@Override
	public void onAuthorDelete(Uri authorUri) {
		//TODO: check that author has no dances before deleting
		final Uri aUri = authorUri;
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setMessage("Are you sure you want to delete this author?")
		       .setCancelable(false)
		       .setPositiveButton("Yes", new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
			       		if(mEditFrag!=null){
			    			FragmentTransaction fTran = mFragManage.beginTransaction();
			    			fTran.remove(mEditFrag);
			    			fTran.commit();
			    			mEditFrag=null;
			    		}
			    		if(mViewFrag!=null){
			    			FragmentTransaction fTran = mFragManage.beginTransaction();
			    			fTran.remove(mViewFrag);
			    			fTran.commit();
			    			mViewFrag=null;
			    		}
	
			            TabAuthor.this.mCr.delete(aUri, null, null);
		           }
		       })
		       .setNegativeButton("No", new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id) {
		                dialog.cancel();
		           }
		       });
		AlertDialog alert = builder.create();
		alert.show();
		
		//mCr.delete(authorUri, null, null);
	}
	
	/**
	 * Handle selection from menu to create a new author
	 */
	@Override
	public void onAuthorNew() {
		if(mEditFrag!=null){
			FragmentTransaction fTran = mFragManage.beginTransaction();
			fTran.remove(mEditFrag);
			fTran.commit();
			mEditFrag=null;
		}
		if(mViewFrag!=null){
			FragmentTransaction fTran = mFragManage.beginTransaction();
			fTran.remove(mViewFrag);
			fTran.commit();
			mViewFrag=null;
		}

		FragmentTransaction fTran = mFragManage.beginTransaction();
		FragAuthorEdit editfrag = new FragAuthorEdit();
		mEditFrag = editfrag;
		fTran.add(R.id.tabauthorfragholder, editfrag, "editfrag");
		fTran.commit();
	}
	
    /**
     * Callback method from edit fragment.  Handles save action.
     */
	@Override
	public void onAuthorUpdate(Long rowId, ContentValues values) {
		// Make sure that there is both a count and a description
		if(values.getAsString(DanceDb.AUTH_FNAME).equals("") || values.getAsString(DanceDb.AUTH_LNAME).equals("")){
			AlertDialog.Builder builder = new AlertDialog.Builder(this);
			builder.setMessage(R.string.author_save_error)
			       .setCancelable(false)
			       .setPositiveButton("OK", new DialogInterface.OnClickListener() {
			           public void onClick(DialogInterface dialog, int id) {
			                dialog.cancel();
			           }
			       });
			AlertDialog alert = builder.create(); 
			alert.show();
		}else{
			if (rowId == null) {
				//Create new author
				try{
					Uri newUri = mCr.insert(DanceProvider.AUTHORS_URI, values);
					rowId = Long.parseLong(newUri.getLastPathSegment().trim());
				}catch(SQLiteConstraintException e){
					Log.d("AuthorEditActivity", "caught SQL problem, couldn't add");
				}
			} else {
				mCr.update(Uri.withAppendedPath(DanceProvider.AUTHORS_URI, String.valueOf(rowId)), 
	        		 values, null, null);
			}
			
			//Remove Edit Fragment and notify list of changes, display view fragment for edited author
			FragmentTransaction fTran = mFragManage.beginTransaction();
			fTran.remove(mEditFrag);
			mEditFrag = null;
			fTran.commit();
			mListFrag.notifyDataChange();
			onAuthorSelected(Uri.withAppendedPath(DanceProvider.AUTHORS_URI, String.valueOf(rowId)));
		}
	}
	

	@Override
	public boolean onNavigationItemSelected(int itemPosition, long itemId) {
		Intent i = null;
		switch(itemPosition) {
	  	        case 0:
	  	        	//Dances
	  	        	break;
	  	        case 1:
	  	        	//Programs
	  	        	break;
	  	        case 2:
	  	        	//Authors
	  	        	break;
	  	        case 3:
	  	        	//Moves
	  	        	i = new Intent(this, TabMove.class);
	  	        	i.addFlags(Intent.FLAG_ACTIVITY_NO_ANIMATION);
	  	        	startActivity(i); 
	  	        	break;
	  	        case 4:
	  	        	//Tags
	  	        	i = new Intent(this, TabTag.class);
	  	        	i.addFlags(Intent.FLAG_ACTIVITY_NO_ANIMATION);
	  	          	startActivity(i);
	  	          	break;
	  	        case 5:
	  	        	//Settings
	  	        	break;
	  	        	
        }	
		return true;
	}



}
