 /*  Copyright 2012 Bronwyn Woods
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package com.dancingwoods.choreo;

import java.util.ArrayList;

import android.app.ActionBar;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.FragmentManager;
import android.app.FragmentTransaction;
import android.content.ContentResolver;
import android.content.ContentValues;
import android.content.DialogInterface;
import android.content.Intent;
import android.database.Cursor;
import android.database.sqlite.SQLiteConstraintException;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.widget.ArrayAdapter;

/**
 * Main activity for dealing with moves in the database.
 * On small screens displays the list fragment of moves 
 * in the database.
 * 
 * @author Bronwyn Woods
 *
 */
public class TabMove extends Activity implements FragMoveView.MoveManipListener, FragMoveList.MoveManipListener, FragMoveEdit.MoveManipListener,
						ActionBar.OnNavigationListener {
	
	private ContentResolver mCr;
	
	private Long mRowId;
    private FragMoveEdit mEditFrag;
    private FragMoveView mViewFrag;
    private FragMoveList mListFrag;
    private FragmentManager mFragManage;
    private ArrayAdapter<CharSequence> mNavSpin;

	
	
	@Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        mCr = getContentResolver();
        setContentView(R.layout.tab_move);
        mFragManage = getFragmentManager();

        //mListFrag = (FragMoveList) mFragManage.findFragmentById(R.id.movelistfrag);
        mEditFrag = null;
 
        // Set up the dropdown navigation menu and set the current selection to moves
        // Use the ActionBar navigation
        mNavSpin = ArrayAdapter.createFromResource(this, R.array.nav_array, 
        		android.R.layout.simple_spinner_item);
        mNavSpin.setDropDownViewResource(R.layout.dropdown_nav);
        ActionBar actionBar = getActionBar();
        actionBar.setNavigationMode(ActionBar.NAVIGATION_MODE_LIST);
        actionBar.setListNavigationCallbacks(mNavSpin, this);
        actionBar.setSelectedNavigationItem(3);
        
        FragmentTransaction fTran = mFragManage.beginTransaction();
		FragMoveList listfrag = new FragMoveList();
		mListFrag = listfrag;
		fTran.add(R.id.tabmovefragholder1, listfrag, "listfrag");
		fTran.commit();
     }
	
	@Override
	public void onResume(){
		super.onResume();
        ActionBar actionBar = getActionBar();
        actionBar.setSelectedNavigationItem(3);
	}
	
	/**
	 * Options menu
	 * new - action bar
	 */
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
	    MenuInflater inflater = getMenuInflater();
	    inflater.inflate(R.menu.tab_move, menu);
	    return true;
	}
    
	/**
	 * Deal with selections from options menu
	 * new - call new activity to enter move
	 */
    @Override
    public boolean onOptionsItemSelected(MenuItem item){
    	//Log.d("OptionsMenu", String.valueOf(item));
        switch(item.getItemId()) {
        case R.id.move_new:
            onMoveNew();
            return true;
//        case android.R.id.home:
//	    	// app icon in action bar clicked; bring 
// 	        Intent intent = new Intent(this, ChoreoActivity.class);
//	        intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
//	        startActivity(intent);
//	        return true;
        }	
        return false;
    }

    /**
     * Callback method to handle selection from move list
     */
	@Override
	public void onMoveSelected(Uri moveUri) {
		if(mEditFrag!=null){
			FragmentTransaction fTran = mFragManage.beginTransaction();
			fTran.remove(mEditFrag);
			fTran.commit();
			mEditFrag=null;
		}
		if(mViewFrag!=null){
			FragmentTransaction fTran = mFragManage.beginTransaction();
			fTran.remove(mViewFrag);
			fTran.commit();
			mViewFrag=null;
		}
		if(mListFrag!=null){
			FragmentTransaction fTran = mFragManage.beginTransaction();
			fTran.remove(mListFrag);
			fTran.commit();
			mListFrag=null;
		}
		FragmentTransaction fTran = mFragManage.beginTransaction();
		FragMoveView viewfrag = new FragMoveView();
    	mRowId = Long.parseLong(moveUri.getLastPathSegment());
    	mViewFrag = viewfrag;
    	fTran.add(R.id.tabmovefragholder1, viewfrag, "viewfrag");
    	//fTran.addToBackStack(null);
		fTran.commit();
		mFragManage.executePendingTransactions();
    	mViewFrag.displayMove(mRowId);

	}

	/**
	 * Callback method to handle edit selected from context menu
	 */
	@Override
	public void onMoveEdit(Uri moveUri) {
		if(mEditFrag!=null){
			FragmentTransaction fTran = mFragManage.beginTransaction();
			fTran.remove(mEditFrag);
			fTran.commit();
			mEditFrag=null;
		}
		if(mViewFrag!=null){
			FragmentTransaction fTran = mFragManage.beginTransaction();
			fTran.remove(mViewFrag);
			fTran.commit();
			mViewFrag=null;
		}
		if(mListFrag!=null){
			FragmentTransaction fTran = mFragManage.beginTransaction();
			fTran.remove(mListFrag);
			fTran.commit();
			mListFrag=null;
		}

		FragmentTransaction fTran = mFragManage.beginTransaction();
		FragMoveEdit editfrag = new FragMoveEdit();
    	mRowId = Long.parseLong(moveUri.getLastPathSegment());
    	mEditFrag = editfrag;
    	fTran.add(R.id.tabmovefragholder1, editfrag, "editfrag");
    	//fTran.addToBackStack(null);
		fTran.commit();
		mFragManage.executePendingTransactions();
    	mEditFrag.displayMove(mRowId);
 	}
	

	/**
	 * Callback method to handle delete from context menu
	 */
	@Override
	public void onMoveDelete(Uri moveUri) {
		mCr.delete(moveUri, null, null);
	}

	/**
	 * Handle selection from menu to create a new move
	 */
	@Override
	public void onMoveNew() {
		FragmentTransaction fTran = mFragManage.beginTransaction();
		FragMoveEdit editfrag = new FragMoveEdit();
		mEditFrag = editfrag;
		fTran.replace(R.id.tabmovefragholder1, editfrag, "editfrag");
    	fTran.addToBackStack(null);
		fTran.commit();
	}
	
    /**
     * Callback method from edit fragment.  Handles save action.
     */
	@Override
	public void onMoveUpdate(Long rowId, ContentValues values, ArrayList<Long> tagids) {
		// Make sure that there is both a count and a description
		if(values.getAsBoolean(DanceDb.MOVE_COUNT)==null || values.getAsBoolean(DanceDb.MOVE_DESC)==null){
			AlertDialog.Builder builder = new AlertDialog.Builder(this);
			builder.setMessage(R.string.move_save_error)
			       .setCancelable(false)
			       .setPositiveButton("OK", new DialogInterface.OnClickListener() {
			           public void onClick(DialogInterface dialog, int id) {
			                dialog.cancel();
			           }
			       });
			AlertDialog alert = builder.create(); 
			alert.show();
		}else{
			if (rowId == null) {
				//Create new move
				try{
					Uri newUri = mCr.insert(DanceProvider.MOVES_URI, values);
					rowId = Long.parseLong(newUri.getLastPathSegment().trim());
				}catch(SQLiteConstraintException e){
					Log.d("MoveEditActivity", "caught SQL problem, couldn't add");
				}
				for(int i=0; i<tagids.size(); i++){
					mCr.insert(Uri.parse(DanceProvider.TAGS_URI+"/"+String.valueOf(tagids.get(i))+"/move/"+rowId), null);
				}

			} else {
				mCr.update(Uri.withAppendedPath(DanceProvider.MOVES_URI, String.valueOf(rowId)), 
	        		 values, null, null);
				// Get the current tags associated with this move
				String[] projection = {DanceDb.MOVETAG_MOVEID, DanceDb.MOVETAG_TAGID};
				Cursor cur = mCr.query(Uri.withAppendedPath(DanceProvider.TAGS_URI, "/move/"+String.valueOf(rowId)), projection, null, null, null);
				// make lists of tags to delete and tags to add
				ArrayList<Long> current = new ArrayList<Long>();
				ArrayList<Long> toadd = new ArrayList<Long>();
				ArrayList<Long> todel = new ArrayList<Long>();
				
				cur.moveToFirst();
				while(!cur.isAfterLast()){
					current.add(Long.valueOf(cur.getLong(1)));
					cur.moveToNext();
				}
				cur.close();
				
				todel = (ArrayList<Long>)current.clone();
				for(int i=0; i<tagids.size(); i++){
					if(current.contains(tagids.get(i))){
						todel.remove(tagids.get(i));
					}else{
						toadd.add(tagids.get(i));
						todel.remove(tagids.get(i));
					}
				}
				
				// Delete tags no longer wanted
				for(int i=0; i<todel.size(); i++){
					mCr.delete(Uri.parse(DanceProvider.TAGS_URI+"/"+String.valueOf(todel.get(i))+"/move/"+String.valueOf(rowId)), null, null);
				}
				// Add new tags
				for(int i=0; i<toadd.size(); i++){
					mCr.insert(Uri.parse(DanceProvider.TAGS_URI+"/"+String.valueOf(toadd.get(i))+"/move/"+rowId), null);
				}

			}
			
			if(mListFrag!=null){
				Log.d("TabMove update", "mListFrag is not null");
				FragmentTransaction fTran = mFragManage.beginTransaction();
				fTran.replace(R.id.tabmovefragholder1, mListFrag, "listfrag");
				fTran.commit();
			}else{
				Log.d("TabMove update", "mListFrag is null");
		        FragmentTransaction fTran = mFragManage.beginTransaction();
				FragMoveList listfrag = new FragMoveList();
				mListFrag = listfrag;
				fTran.add(R.id.tabmovefragholder1, listfrag, "listfrag");
				fTran.commit();
			}
		}
	}
	

	@Override
	public boolean onNavigationItemSelected(int itemPosition, long itemId) {
		Intent i = null;
		switch(itemPosition) {
	  	        case 0:
	  	        	//Dances
	  	        	break;
	  	        case 1:
	  	        	//Programs
	  	        	break;
	  	        case 2:
	  	        	//Authors
	  	        	i = new Intent(this, TabAuthor.class);
	  	        	i.addFlags(Intent.FLAG_ACTIVITY_NO_ANIMATION);
	  	          	startActivity(i);
	  	          	break;
	  	        case 3:
	  	        	//Moves
	  	        	break;
	  	        case 4:
	  	        	//Tags
	  	        	i = new Intent(this, TabTag.class);
	  	        	i.addFlags(Intent.FLAG_ACTIVITY_NO_ANIMATION);
	  	          	startActivity(i);
	  	          	break;
	  	        case 5:
	  	        	//Settings
	  	        	break;
	  	        	
        }	
		return true;
	}



}
