/*  Copyright 2012 Bronwyn Woods
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package com.dancingwoods.choreo;

import android.app.ActionBar;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.FragmentManager;
import android.app.FragmentTransaction;
import android.content.ContentResolver;
import android.content.ContentValues;
import android.content.DialogInterface;
import android.content.Intent;
import android.database.sqlite.SQLiteConstraintException;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.widget.ArrayAdapter;

/**
 * Main activity for dealing with tags in the database.
 * 
 * @author Bronwyn Woods
 *
 */
public class TabTag extends Activity implements ActionBar.OnNavigationListener, FragTagList.TagManipListener,
	FragTagEdit.TagManipListener{
	
	private ContentResolver mCr;
	
	private Long mRowId;
    private FragTagList mListFrag;
    private FragTagDance mDanceFrag;
    private FragTagMove mMoveFrag;
    private FragTagEdit mEditFrag;
    
    private FragmentManager mFragManage;
    private ArrayAdapter<CharSequence> mNavSpin;

	
	@Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        mCr = getContentResolver();
        setContentView(R.layout.tab_tag);
        mFragManage = getFragmentManager();

        mListFrag = (FragTagList) mFragManage.findFragmentById(R.id.taglistfrag);
        mDanceFrag = null;
        mMoveFrag = null;
        mEditFrag = null;
 
        // Set up the dropdown navigation menu and set the current selection to tags
        // Use the ActionBar navigation
        mNavSpin = ArrayAdapter.createFromResource(this, R.array.nav_array, 
        		android.R.layout.simple_spinner_item);
        mNavSpin.setDropDownViewResource(R.layout.dropdown_nav);
        ActionBar actionBar = getActionBar();
        actionBar.setNavigationMode(ActionBar.NAVIGATION_MODE_LIST);
        actionBar.setListNavigationCallbacks(mNavSpin, this);
        actionBar.setSelectedNavigationItem(4);
    }
	
	@Override
	public void onResume(){
		super.onResume();
        ActionBar actionBar = getActionBar();
        actionBar.setSelectedNavigationItem(4);
	}
	
	/**
	 * Options menu
	 * new - action bar
	 */
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
	    MenuInflater inflater = getMenuInflater();
	    inflater.inflate(R.menu.tab_tag, menu);
	    return true;
	}

	/**
	 * Deal with selections from options menu
	 * new - call new activity to enter author
	 */
    @Override
    public boolean onOptionsItemSelected(MenuItem item){
    	//Log.d("OptionsMenu", String.valueOf(item));
        switch(item.getItemId()) {
        case R.id.tag_new:
            onTagNew();
            return true;
        }	
        return false;
    }

	/**
	 * Handle selection from menu to create a new tag
	 */
	@Override
	public void onTagNew() {
		if(mDanceFrag!=null){
			FragmentTransaction fTran = mFragManage.beginTransaction();
			fTran.remove(mEditFrag);
			fTran.commit();
			mDanceFrag=null;
		}
		if(mMoveFrag!=null){
			FragmentTransaction fTran = mFragManage.beginTransaction();
			fTran.remove(mMoveFrag);
			fTran.commit();
			mMoveFrag=null;
		}
		if(mEditFrag!=null){
			FragmentTransaction fTran = mFragManage.beginTransaction();
			fTran.remove(mEditFrag);
			fTran.commit();
			mEditFrag=null;
		}

		FragmentTransaction fTran = mFragManage.beginTransaction();
		FragTagEdit editfrag = new FragTagEdit();
		mEditFrag = editfrag;
		fTran.add(R.id.tabtagfragholder1, editfrag, "editfrag");
		fTran.commit();
	}



	@Override
	public void onTagSelected(Uri tagUri) {
		// TODO Auto-generated method stub
		Log.d("Tag selected",String.valueOf(tagUri));
		
	}


	@Override
	public void onTagEdit(Uri tagUri) {
		// Remove any existing fragments in the right hand panel
		if(mEditFrag!=null){
			FragmentTransaction fTran = mFragManage.beginTransaction();
			fTran.remove(mEditFrag);
			fTran.commit();
			mEditFrag=null;
		}
		if(mDanceFrag!=null){
			FragmentTransaction fTran = mFragManage.beginTransaction();
			fTran.remove(mDanceFrag);
			fTran.commit();
			mDanceFrag=null;
		}
		if(mMoveFrag!=null){
			FragmentTransaction fTran = mFragManage.beginTransaction();
			fTran.remove(mMoveFrag);
			fTran.commit();
			mMoveFrag=null;
		}

		FragmentTransaction fTran = mFragManage.beginTransaction();
		FragTagEdit editfrag = new FragTagEdit();
    	mRowId = Long.parseLong(tagUri.getLastPathSegment());
    	mEditFrag = editfrag;
    	fTran.add(R.id.tabtagfragholder1, editfrag, "editfrag");
		fTran.commit();
		mFragManage.executePendingTransactions();
    	mEditFrag.displayTag(mRowId);
		
	}


	@Override
	public void onTagDelete(Uri tagUri) {
		mCr.delete(tagUri, null, null);
	}

	
    /**
     * Callback method from edit fragment.  Handles save action.
     */
	@Override
	public void onTagUpdate(Long rowId, ContentValues values) {
		// Make sure that there is both a category and description
		if(values.getAsString(DanceDb.TAG_DESC).equals("") || values.getAsString(DanceDb.TAG_CAT).equals("")){
			AlertDialog.Builder builder = new AlertDialog.Builder(this);
			builder.setMessage(R.string.tag_save_error)
			       .setCancelable(false)
			       .setPositiveButton("OK", new DialogInterface.OnClickListener() {
			           public void onClick(DialogInterface dialog, int id) {
			                dialog.cancel();
			           }
			       });
			AlertDialog alert = builder.create(); 
			alert.show();
		}else{
			if (rowId == null) {
				//Create new author
				try{
					Uri newUri = mCr.insert(DanceProvider.TAGS_URI, values);
					rowId = Long.parseLong(newUri.getLastPathSegment().trim());
				}catch(SQLiteConstraintException e){
					Log.d("TagEditActivity", "caught SQL problem, couldn't add");
				}
			} else {
				mCr.update(Uri.withAppendedPath(DanceProvider.TAGS_URI, String.valueOf(rowId)), 
	        		 values, null, null);
			}
			
			//Remove Edit Fragment and notify list of changes, display view fragment for edited author
			FragmentTransaction fTran = mFragManage.beginTransaction();
			fTran.remove(mEditFrag);
			mEditFrag = null;
			fTran.commit();
			mListFrag.notifyDataChange();
			//onTagSelected(Uri.withAppendedPath(DanceProvider.AUTHORS_URI, String.valueOf(rowId)));
		}
	}

	@Override
	public boolean onNavigationItemSelected(int itemPosition, long itemId) {
		Intent i = null;
		switch(itemPosition) {
	  	        case 0:
	  	        	//Dances
	  	        	break;
	  	        case 1:
	  	        	//Programs
	  	        	break;
	  	        case 2:
	  	        	//Authors
	  	        	i = new Intent(this, TabAuthor.class);
	  	        	i.addFlags(Intent.FLAG_ACTIVITY_NO_ANIMATION);
	  	          	startActivity(i);
	  	        	break;
	  	        case 3:
	  	        	//Moves
	  	        	i = new Intent(this, TabMove.class);
	  	        	i.addFlags(Intent.FLAG_ACTIVITY_NO_ANIMATION);
	  	        	startActivity(i); 
	  	        	break;
	  	        case 4:
	  	        	//Tags
	  	          	break;
	  	        case 5:
	  	        	//Settings
	  	        	break;
	  	        	
        }	
		return true;
	}



}
